#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>
#include <string.h>

#include "lzw.h"

int
main(int argc, char *argv[])
{
	struct lzw_decoder dec;
	unsigned char buffer[65536];
	unsigned char buffer2[65536];
	const unsigned char *in;
	unsigned char *out;
	size_t count, insz, outsz;
	size_t nread, nwrite;
	int rc;

	nread = nwrite = 0;

	lzw_decoder_init(&dec);

	while ((count = read(0, buffer, sizeof(buffer))) > 0) {

		in = buffer;
		insz = count;
		nread += count;

		while (insz > 0) {

			out = buffer2;
			outsz = sizeof(buffer2);
			rc = lzw_decode(&dec, &in, &insz, &out, &outsz);

			if (rc == LZW_OK || rc == LZW_NOSPACE) {
				count = out - buffer2;
				rc = write(1, buffer2, count);
				if (rc == -1) {
					fprintf(stderr, "write error: %s\n", strerror(errno));
					exit(2);
				}
				nwrite += count;
			} else {

				if (rc == LZW_NOMEM)
					fprintf(stderr, "out of memory\n");
				else
					fprintf(stderr, "unknown error %d\n", rc);
				exit(2);
			}
		}
	}

	lzw_decoder_deinit(&dec);

	fprintf(stderr, "nread %lu, nwrite %lu\n", nread, nwrite);

	return 0;
}

